// ------------------------------------------------------------------
// da_except.h
// written by:    [PUT YOUR NAME HERE]
// last modified: [PUT THE DUE DATE HERE]
//
// Description:
// [UPDATE this description, delete most of the below,
//      replace it with something more descriptive
//      of what the classes in this file are intended to do
//      like: This file declares and defines classess
//            to provide a way to report errors in
//            try-catch situations.
// ]
//
// This file sort of bends/breaks some general rules about 
// what you should and should not do. 
// You will encounter this ALOT in the real world.
// Understanding why this is ok and why it may not be ok
// is important.
//
// What is going on here is simple, and complicated.
// It demonstrates many things you can do with classes.
// Do not worry too much if you do not yet understand what
// exactly is going on here. You will in time.  =)
//
// It is both good and bad.
// It allows something to be done in a fairly easy way.
// It is also pretty easy to maintain and add to if needed.
// It has the potential to really mess some things up.
//
// AND it is both defining and implementing not only 1 class
// but many classes in a header file.
//
// If not done REALLY CAREFULLY and with UNDERSTANDING
// bad things could happen.
//
// Explaining that a little better:
//
// General rules/guidelines of good programming say
// Header files are for interface definition
// CPP files are for implementation
// One header and one CPP file for each class.
//
// NOTE: in all the classes below they ONLY have constructor
//       functions. This sort of minimizes the breakage of
//       the general guidelines
//       As some people will implement the constructors of
//       all their classes in the header files as this
//       when done properly allows those using the interface
//       to see the default/initial values of things.
//       Notice HOW the constructors are being implemented.
//
// All is ok as long as what is being done stays unchanged
// and when changed only does things in a very similar,
// if not exactly the same, way.
//
// But... special cases always come up and the temptation
// will be to put things in this file that don't belong
// Like other more complicated class member functions...
// and after a while everything gets cluttered with
// strange behaviors and odd errors... but only when other
// errors are happenning... making it very difficult to debug
//
// The point: Always be careful with your code.
// Be absolutely certain you understand why something is
// or is not a good idea. Do not follow blindly.
// If something seems like a bad idea, it very well may be.
// See if there is an explanatino for why it is ok.
// Test the explanation.
// Assume others will challenge you in a similar manner.
// Approach all such discussions as learning opportunities
// It is possible both sides may be right (or both wrong),
// or there are factors beyond the code itself.
// However, for this class, your instructor is ALWAYS right.  =)
//
// ------------------------------------------------------------------

#pragma once

// #ifndef EXCEPTION_CLASSES   // #pragma once makes this no longer needed
// #define EXCEPTION_CLASSES   // #pragma once makes this no longer needed

#include <sstream>
#include <string>

using namespace std;

// ------------------------------------------------------------------
// A baseException class  
// ---- do NOT confuse this with std::exception
// ------------------------------------------------------------------
class baseException
{
   public:
      baseException(const string& str = ""):
         msgString(str)
      {
         if (msgString == "")
            msgString = "Unspecified exception";
      }

      string what() const
      {
         return msgString;
      }

   // protected allows a derived class to access msgString.
   // chapter 13 (of a previous book used for this class)
   // discusses protected in detail
   // For now (this assignement) you need not worry about it.
   protected:
      string msgString;
};

// ------------------------------------------------------------------
// failure to allocate memory (i.e. new [something] returned a NULL)
// ------------------------------------------------------------------
class memoryAllocationError: public baseException
{
   public:
      memoryAllocationError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// function argument out of proper range
// ------------------------------------------------------------------
class rangeError: public baseException
{
   public:
      rangeError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// index out of range
// ------------------------------------------------------------------
class indexRangeError: public baseException
{
   public:
      indexRangeError(const string& msg, int i, int size):
         baseException()
      {
         ostringstream indexErr;

         indexErr << msg << "  index " << i << "  size = " << size << ends;
         // indexRangeError can modify msgString, since it is in
         // the protected section of baseException
         msgString = indexErr.str();
      }
};

// ------------------------------------------------------------------
// attempt to erase from an empty container
// ------------------------------------------------------------------
class underflowError: public baseException
{
   public:
      underflowError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// attempt to insert into a full container
// ------------------------------------------------------------------
class overflowError: public baseException
{
   public:
      overflowError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// error in expression evaluation
// ------------------------------------------------------------------
class expressionError: public baseException
{
   public:
      expressionError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// bad object reference
// ------------------------------------------------------------------
class referenceError: public baseException
{
   public:
      referenceError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// feature not implemented
// ------------------------------------------------------------------
class notImplementedError: public baseException
{
   public:
      notImplementedError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// date errors
// ------------------------------------------------------------------
class dateError: public baseException
{
   public:
      dateError(const string& first, int v, const string& last):
         baseException()
      {
         ostringstream dateErr;

         dateErr << first << ' ' << v << ' ' << last << ends;
         // dateError can modify msgString, since it is in
         // the protected section of baseException
         msgString = dateErr.str();
      }
};

// ------------------------------------------------------------------
// error in graph class
// ------------------------------------------------------------------
class graphError: public baseException
{
   public:
      graphError(const string& msg = ""):
         baseException(msg)
      {}
};

// ------------------------------------------------------------------
// file open error
// ------------------------------------------------------------------
class fileOpenError: public baseException
{
   public:
      fileOpenError(const string& fname):
         baseException()
      {
         ostringstream fileErr;

         fileErr << "Cannot open \"" << fname << "\"" << ends;
         // fileOpenError can modify msgString, since it is in
         // the protected section of baseException
         msgString = fileErr.str();
      }
};

// ------------------------------------------------------------------
// error in graph class
// ------------------------------------------------------------------
class fileError: public baseException
{
   public:
      fileError(const string& msg = ""):
         baseException(msg)
      {}
};

// #endif     // EXCEPTION_CLASSES   --- // #pragma once makes this no longer needed
